--=============================================================================--
--===========================================================================--
library ieee;
   use ieee.std_logic_1164.all;
   use IEEE.STD_LOGIC_ARITH.ALL;
   use IEEE.STD_LOGIC_UNSIGNED.ALL;
   use ieee.numeric_std.all;

entity System09_Terasic_DE1 is
  port(
	-- clocks
	clock_24    : in    std_logic_vector(1 downto 0);
	clock_27    : in    std_logic; 
    clock_50    : in    std_logic;  -- System Clock input
    ext_clock   : in    std_logic;
    
    -- push button keys
	key         : in    std_logic_vector(3 downto 0);  -- RST, NMI

	-- LEDS & Switches
	ledg        : out std_logic_vector(7 downto 0);
	ledr        : out std_logic_vector(9 downto 0);
	sw          : in  std_logic_vector(9 downto 0);
	
	--Analog
	gpio_0      : out   std_logic_vector(17 downto 0);
	gpio_1      : out   std_logic_vector(17 downto 0);
	
	--Hex
	hex0	    : out std_logic_vector (0 to 6);
	hex1	    : out std_logic_vector (0 to 6);
	hex2	    : out std_logic_vector (0 to 6);
	hex3	    : out std_logic_vector (0 to 6)
	
	);
end System09_Terasic_DE1;

-------------------------------------------------------------------------------
-- Architecture for System09
-------------------------------------------------------------------------------
architecture my_computer of System09_Terasic_DE1 is
  -----------------------------------------------------------------------------
  -- constants
  -----------------------------------------------------------------------------
  constant SYS_CLK_FREQ  : integer := 50000000;  -- FPGA System Clock
  constant VGA_CLK_FREQ  : integer := 25000000;  -- VGA Pixel Clock
  constant CPU_CLK_FREQ  : integer := 12000000;  -- CPU Clock

  type hold_state_type is ( hold_release_state, hold_request_state );

  -----------------------------------------------------------------------------
  -- Signals
  -----------------------------------------------------------------------------

  signal sys_clk       	: std_logic; 
  signal vga_clk       	: std_logic;
  signal pll_locked    	: std_logic;
  signal resetsr 		: std_logic_vector(1 downto 0);
  signal ares			: std_logic;
  signal cnt1m5			: integer range 0 to 7; 
  signal clk1m5			: std_logic;
  signal cnt6m			: integer range 0 to 1; 
  signal clk6m			: std_logic;
  signal cnt750k		: integer range 0 to 7; 
  signal clk750k		: std_logic;

  
  -- CPU Interface signals
  signal cpu_clk       : std_logic;
  signal cpu_rst       : std_logic;
  signal cpu_rw        : std_logic;
  signal cpu_vma       : std_logic;
  signal cpu_halt      : std_logic;
  signal cpu_hold      : std_logic;
  signal cpu_firq      : std_logic;
  signal cpu_irq       : std_logic;
  signal cpu_nmi       : std_logic;
  signal cpu_addr      : std_logic_vector(15 downto 0);
  signal cpu_data_in   : std_logic_vector(7 downto 0);
  signal cpu_data_out  : std_logic_vector(7 downto 0);
  signal cpu_pc		   : std_logic_vector(15 downto 0);

  -- BOOT ROM
  signal rom_cs        : Std_logic;
  signal rom_data_out  : Std_Logic_Vector(7 downto 0);

  -- IO / VIA
  signal io_cs       : std_logic;
  signal io_data_out : std_logic_vector(7 downto 0);
  signal io_data_oe  : std_logic;
  signal via_wen	 : std_logic;
  signal via_ren     : std_logic;
  signal via_pa_out  : std_logic_vector(7 downto 0);
  signal via_pa_oe	 : std_logic_vector(7 downto 0);
  signal via_pa_in   : std_logic_vector(7 downto 0);
  signal via_pb_out  : std_logic_vector(7 downto 0);
  signal via_pb_oe	 : std_logic_vector(7 downto 0);
  signal via_pb_in	 : std_logic_vector(7 downto 0);
  signal via_irq_out : std_logic; -- ?????????????
  signal via_ca1_in  : std_logic; -- to IO7 AY8912
  signal via_ca2_out : std_logic; -- alg n_zero
  signal via_ca2_oe  : std_logic; -- alg n_zero
  signal via_ca2_in  : std_logic; -- alg n_zero
  signal via_cb1_out : std_logic; -- nc
  signal via_cb1_oe  : std_logic; -- nc
  signal via_cb1_in  : std_logic; -- nc
  signal via_cb2_out : std_logic; -- alg n_blank
  signal via_cb2_oe  : std_logic; -- alg n_blank
  signal via_cb2_in  : std_logic; -- alg n_blank
  
  -- Sound / AY
  signal ay_bc1		 : std_logic; -- to AY bc1
  signal ay_bdir     : std_logic; -- to AY bdir
  signal ay_pa_out	 : std_logic_vector(7 downto 0);
  signal ay_data_out : std_logic_vector(7 downto 0);
  signal ay_data_in  : std_logic_vector(7 downto 0);
  signal ay_data_oe  : std_logic;
  signal analog_sound    : std_logic_vector(7 downto 0);
  
  -- CART
  signal cart_cs       : std_logic;
  signal cart_data_out : std_logic_vector(7 downto 0);

  -- RAM
  signal sram_cs       : std_logic;
  signal sram_data_out : std_logic_vector(7 downto 0);
  signal sram_ce       : std_logic;
  signal sram_ub       : std_logic;
  signal sram_lb       : std_logic;
  signal sram_we       : std_logic;
  signal sram_oe       : std_logic;
  
  --Analog
  signal analog_vec    : std_logic_vector(7 downto 0);
  signal analog_x      : std_logic_vector(7 downto 0);
  signal analog_y      : std_logic_vector(7 downto 0);
  signal analog_y_temp : std_logic_vector(7 downto 0);
  signal analog_x_int  : std_logic_vector(7 downto 0);
  signal analog_y_int  : std_logic_vector(7 downto 0);
  signal analog_z      : std_logic_vector(7 downto 0);
  signal analog_z_temp : std_logic_vector(7 downto 0);
  signal analog_offs   : std_logic_vector(7 downto 0);
  signal cs_x          : std_logic;
  signal cs_y          : std_logic;
  signal cs_x_int      : std_logic;
  signal cs_y_int      : std_logic;
  signal cs_z          : std_logic;


-----------------------------------------------------------------
--
-- CPU09 CPU core
--
-----------------------------------------------------------------

component cpu09
  port (    
	 clk      :	in  std_logic;
     rst      : in  std_logic;
     vma      :	out std_logic;
     addr     : out std_logic_vector(15 downto 0);
     rw       :	out std_logic;
     data_in  : in	 std_logic_vector(7 downto 0);
	 data_out : out std_logic_vector(7 downto 0);
	 halt     : in  std_logic;
	 hold     : in  std_logic;
	 irq      : in  std_logic;
	 nmi      : in  std_logic;
	 firq     : in  std_logic;
	 dpc      : out std_logic_vector (15 downto 0)
  );
end component;

----------------------------------------
--
-- 8KByte Block RAM as Main ROM
--
----------------------------------------
component mon_rom
  Port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    cs       : in  std_logic;
    rw       : in  std_logic;
    addr     : in  std_logic_vector (12 downto 0);
    data_in  : in  std_logic_vector (7 downto 0);
    data_out : out std_logic_vector (7 downto 0)
    );
end component;

component cart_rom
  Port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    cs       : in  std_logic;
    rw       : in  std_logic;
    addr     : in  std_logic_vector (12 downto 0);
    data_in  : in  std_logic_vector (7 downto 0);
    data_out : out std_logic_vector (7 downto 0)
    );
end component;

component ram_2k
 Port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    cs       : in  std_logic;
    rw       : in  std_logic;
    addr     : in  std_logic_vector (9 downto 0);
    data_in  : in  std_logic_vector (7 downto 0);
    data_out : out std_logic_vector (7 downto 0)
    );
end component;

component pll IS
	PORT
	(
		areset		: IN STD_LOGIC  := '0';
		inclk0		: IN STD_LOGIC  := '0';
		c0			: OUT STD_LOGIC ;
		c1			: OUT STD_LOGIC ;
		c2			: OUT STD_LOGIC ;
		locked		: OUT STD_LOGIC 
	);
end component;


component via6522 is
port (
    clock       : in  std_logic;
    clock_en    : in  std_logic; -- for counters and stuff
    reset       : in  std_logic;
    
    addr        : in  std_logic_vector(3 downto 0);
    wen         : in  std_logic;
    ren         : in  std_logic;
    data_in     : in  std_logic_vector(7 downto 0);
    data_out    : out std_logic_vector(7 downto 0);

    -- pio --
    port_a_o    : out std_logic_vector(7 downto 0);
    port_a_t    : out std_logic_vector(7 downto 0);
    port_a_i    : in  std_logic_vector(7 downto 0);
    
    port_b_o    : out std_logic_vector(7 downto 0);
    port_b_t    : out std_logic_vector(7 downto 0);
    port_b_i    : in  std_logic_vector(7 downto 0);

    -- handshake pins
    ca1_i       : in  std_logic;

    ca2_o       : out std_logic;
    ca2_i       : in  std_logic;
    ca2_t       : out std_logic;
    
    cb1_o       : out std_logic;
    cb1_i       : in  std_logic;
    cb1_t       : out std_logic;
    
    cb2_o       : out std_logic;
    cb2_i       : in  std_logic;
    cb2_t       : out std_logic;

    irq         : out std_logic
      );
end component;

component PWM is
      Port ( 
            clk      : in  std_logic;
            pwmvalue : in  std_logic_vector (7 downto 0);
            pwmout   : out std_logic
            );
end component;

component SPRITE_PWM is
      Port ( 
           clk      	: in  STD_LOGIC;
           pwmvalue 	: in  STD_LOGIC_VECTOR (7 downto 0);
           pwm_stb  	: in std_logic;
           pwmout_h   	: out STD_LOGIC;
           pwmout_l   	: out STD_LOGIC
            );
end component;

component signed_integ is
      Port ( 
			  clk_i              :  in  std_logic;
			  rst_i              :  in  std_logic;
			  ramp_i			 :  in  std_logic;
			  zero_i			 :  in  std_logic;
			  load_offset_i		 :  in  std_logic;
			  data_i          	 :  in  std_logic_vector(7 downto 0);
			  offset_i			 :  in  std_logic_vector(7 downto 0);
			  data_o        	 :  out std_logic_vector(7 downto 0);
			  data_o_stb		 :  out std_logic
            );
end component;

component hexdisplay is
    Port( code  	:  in STD_LOGIC_VECTOR (3 downto 0);
          seg7   	:  out STD_LOGIC_VECTOR (0 to 6)
         );
end component;

component YM2149 is
	Port (
		  -- data bus
		  I_DA                : in  std_logic_vector(7 downto 0);
		  O_DA                : out std_logic_vector(7 downto 0);
		  O_DA_OE_L           : out std_logic;
		  -- control
		  I_A9_L              : in  std_logic;
		  I_A8                : in  std_logic;
		  I_BDIR              : in  std_logic;
		  I_BC2               : in  std_logic;
		  I_BC1               : in  std_logic;
		  I_SEL_L             : in  std_logic;

		  O_AUDIO             : out std_logic_vector(7 downto 0);
		  -- port a
		  I_IOA               : in  std_logic_vector(7 downto 0);
		  O_IOA               : out std_logic_vector(7 downto 0);
		  O_IOA_OE_L          : out std_logic;
		  -- port b
		  I_IOB               : in  std_logic_vector(7 downto 0);
		  O_IOB               : out std_logic_vector(7 downto 0);
		  O_IOB_OE_L          : out std_logic;
		  --
		  ENA                 : in  std_logic; -- clock enable
		  RESET_L             : in  std_logic; -- E clock , 1.5MHz
		  CLK                 : in  std_logic  -- 6 Mhz
		);
end component;


begin
  -----------------------------------------------------------------------------
  -- Instantiation of internal components
  -----------------------------------------------------------------------------

my_cpu : cpu09  port map (    
	 clk	   => not clk6m,
	 rst       => cpu_rst,
     vma       => cpu_vma,
     addr      => cpu_addr(15 downto 0),
     rw	       => cpu_rw,
     data_in   => cpu_data_in,
	 data_out  => cpu_data_out,
	 halt      => '0',--cpu_halt,
	 hold      => '0',--cpu_hold,
	 irq       => '0', --not via_irq_out,
	 nmi       => '0', --cpu_nmi,
	 firq      => '0',--ay_pa_out(3),--cpu_firq,
	 dpc	   => cpu_pc
    );

my_rom : mon_rom port map (
    clk       => clk6m,
    rst       => cpu_rst,
	cs        => rom_cs,
	rw        => '1',
    addr      => cpu_addr(12 downto 0),
    data_in   => cpu_data_out,
    data_out  => rom_data_out
    );
    
my_cart : cart_rom port map (
    clk       => clk6m,
    rst       => cpu_rst,
	cs        => cart_cs,
	rw        => '1',
    addr      => cpu_addr(12 downto 0),
    data_in   => cpu_data_out,
    data_out  => cart_data_out
    );
    
my_ram : ram_2k port map (
	clk       => clk6m,
    rst       => cpu_rst,
	cs        => sram_cs,
	rw        => cpu_rw,
    addr      => cpu_addr(9 downto 0),
    data_in   => cpu_data_out,
    data_out  => sram_data_out
	);

my_via : via6522 port map (
    
		clock       => clk6m,
        clock_en    => clk1m5,
        reset       => cpu_rst,
                                
        addr        => cpu_addr(3 downto 0),
        wen         => not cpu_rw and io_cs and cpu_addr(12),
        ren         => cpu_rw and io_cs and cpu_addr(12),
        data_in     => cpu_data_out,
        data_out    => io_data_out,
                                
        -- pio --   
        port_a_o    => via_pa_out,
        port_a_t    => via_pa_oe,
        port_a_i    => via_pa_in,
                                
        port_b_o    => via_pb_out,
        port_b_t    => via_pb_oe,
        port_b_i    => via_pb_in,
    
        -- handshake pins
        ca1_i       => ay_pa_out(7),--via_ca1_in,
                            
        ca2_o       => via_ca2_out,
        ca2_i       => via_ca2_in,
        ca2_t       => via_ca2_oe,
                            
        cb1_o       => via_cb1_out,
        cb1_i       => via_cb1_in,
        cb1_t       => via_cb1_oe,
                            
        cb2_o       => via_cb2_out,
        cb2_i       => via_cb2_in,
        cb2_t       => via_cb2_oe,
                            
        irq         => via_irq_out
    );

    
my_ay : YM2149
	port map (
	-- data bus
    I_DA                => ay_data_in,
    O_DA                => ay_data_out,
    O_DA_OE_L           => ay_data_oe,
    -- control
    I_A9_L              => '0',
    I_A8                => '1',
    I_BDIR              => ay_bdir,
    I_BC2               => '1',
    I_BC1               => ay_bc1,
    I_SEL_L             => '1',

    O_AUDIO             => analog_sound,
    -- port a
    I_IOA               => "11111111",	--joysticks
    O_IOA               => ay_pa_out,
    O_IOA_OE_L          => open,		--ioa_oe_l,
    -- port b
    I_IOB               => "00000000", 	--portb nc at AY38912
    O_IOB               => open, 		--portb nc at AY38912
    O_IOB_OE_L          => open, 		--portb nc at AY38912

    ENA                 => '1',
    RESET_L             => not cpu_rst,
    CLK                 => clk1m5
    );


x_int : signed_integ
	port map (
			clk_i      		=> clk6m,
			rst_i      		=> cpu_rst,
			ramp_i	   		=> not via_pb_out(7),
			zero_i	   		=> not via_ca2_out,
			load_offset_i	=> '1',
			data_i			=> analog_x(7 downto 0),
			offset_i		=> analog_offs(7 downto 0),
			data_o			=> analog_x_int(7 downto 0),
			data_o_stb		=> cs_x_int
			);
			
y_int : signed_integ
	port map (
			clk_i      		=> clk6m,
			rst_i      		=> cpu_rst,
			ramp_i	   		=> not via_pb_out(7),
			zero_i	   		=> not via_ca2_out,
			load_offset_i	=> '1',
			data_i			=> analog_y(7 downto 0),
			offset_i		=> analog_offs(7 downto 0),
			data_o			=> analog_y_int(7 downto 0),
			data_o_stb		=> cs_y_int
			);

sound_pwm : PWM
   port map( CLK => cpu_clk,
             pwmvalue => analog_sound,
			 pwmout => gpio_1(0)
           );
           
x_pwm : SPRITE_PWM
   port map( CLK => sys_clk,
             pwmvalue => analog_x_int(7 downto 0),
             pwm_stb  => clk6m,
			 pwmout_h => gpio_1(1),
			 pwmout_l => gpio_1(2)
           );

y_pwm : SPRITE_PWM
   port map( CLK => sys_clk,
             pwmvalue => analog_y_int(7 downto 0),
             pwm_stb  => clk6m,
			 pwmout_h => gpio_1(3),
			 pwmout_l => gpio_1(4)
           );

z_pwm : SPRITE_PWM
   port map( CLK => sys_clk,
             pwmvalue => analog_z,
             pwm_stb  => '1'                                                             ,
			 pwmout_h => gpio_1(5),
			 pwmout_l => gpio_1(6)
           );

chex0 : hexdisplay
	port map(
			code => cpu_pc( 3 downto 0),
			seg7 => hex0(0 to 6)
			);
chex1 : hexdisplay
	port map(
			code => cpu_pc( 7 downto 4),
			seg7 => hex1(0 to 6)
			);
chex2 : hexdisplay
	port map(
			code => cpu_pc( 11 downto 8),
			seg7 => hex2(0 to 6)
			);
chex3 : hexdisplay
	port map(
			code => cpu_pc( 15 downto 12),
			seg7 => hex3(0 to 6)
			);

----------------------------------------
--
-- Phase Locked Loop Clock divider
--

my_pll : pll
	PORT MAP
	(
		areset		=> '0',
		inclk0		=> clock_50,
		c0			=> sys_clk,
		c1			=> cpu_clk,
		c2			=> vga_clk,
		locked		=> pll_locked 
	);
	 
----------------------------------------------------------------------
--
-- Process to decode memory map
--
----------------------------------------------------------------------

mem_decode: process(cpu_rst,cpu_data_out,key,resetsr,cpu_addr,cart_data_out,sw,
					cpu_vma,sram_data_out,io_data_out,rom_data_out)
begin
--	if sys_clk'event and sys_clk='1' then

		  cpu_rst    <= resetsr(1); -- CPU reset is active high	  
		  ares		 <= not key(0);		
		
		if cpu_rst = '1' then
		cpu_data_in <= (others=>'Z');
		  cpu_data_in <= cpu_data_out;
		  rom_cs      <= '0';
		  io_cs       <= '0';
		  sram_cs     <= '0';
		  cart_cs	  <= '0';      
		  cpu_halt   <= '0';
		  cpu_hold   <= not key(2);
		  cpu_firq   <= '0';   
		  cpu_irq    <= '0';   
		  cpu_nmi    <= not key(1);
--		  cpu_rst    <= resetsr(1); -- CPU reset is active high	  
--		  ares		 <= not key(0);
		  else
		 

	-- Memory Decoder

		if cpu_addr( 15 ) = '0' then 				--CART 0x0000-0x7FFF
			cpu_data_in <= cart_data_out;
			
			if sw(1) = '1' then
				cpu_data_in <= "00000000";
			else
				cart_cs     <= cpu_vma;
			end if;
		else
			cart_cs <= '0';
		end if;  

		if cpu_addr( 15 downto 11 ) = "11001" then 	--RAM 0xC800-0xCFFF
		--if cpu_addr( 15 downto 12 ) = "1100" then 	--RAM 0xC800-0xCFFF
			cpu_data_in <= sram_data_out;
			sram_cs     <= cpu_vma;
		else
			sram_cs <= '0';
		end if;
		
		if cpu_addr( 15 downto 12 ) = "1101" then 	--IO 0xD000-0xDFFF	    
			cpu_data_in <= io_data_out;
			io_cs     <= cpu_vma;
		else
			io_cs <= '0';	
		end if;

		if cpu_addr( 15 downto 13 ) = "111" then 	--ROM 0xE000-0xFFFF
			cpu_data_in <= rom_data_out;
			rom_cs      <= cpu_vma;
		else
			rom_cs <= '0';
		end if;
		
  end if;      
end process;


-- LEDS output register

led_output : process( via_cb1_out , sram_cs , rom_cs , cart_cs , io_cs )
begin
	ledr(9) <= cart_cs; 
	ledr(8) <= io_cs; 
	ledr(7) <= sram_cs;
	ledr(6) <= rom_cs;
	ledr(5) <= via_cb1_out;
end process;

led_output2 : process( clk1m5, cpu_rst, sram_cs, cpu_rw, cpu_addr, cpu_data_out)
begin
	if clk1m5'event and clk1m5='0' then
		if cpu_rst = '1' then
			ledg <= (others=>'0');
		else
			ledg(7 downto 0) <= cpu_addr(15 downto 8);
		end if;
	end if;
end process;


update_alg : process( clk6m, clk1m5 ,cpu_rst,via_pb_out,via_cb1_out,via_pb_oe,via_ca2_out,
					via_cb2_out,analog_vec,analog_y_temp,via_pa_oe,ay_data_oe,ay_data_out,
					analog_x,via_pa_out)

-- Resolution: 330  410

begin	
--	if clk6m'event and clk6m= '1' then
		if cpu_rst = '1' then
		
			ay_bc1		   			<= '0';
			ay_bdir        			<= '0';
			gpio_0(2)				<= not '0';
			gpio_0(3)				<= not '0';
			gpio_0(4)				<= '0';
			gpio_0(6)				<= '0';
			gpio_0(7)				<= '0';
			gpio_0(8)				<= '0';
			gpio_0(17 downto 10)	<= "00000000";
			via_pa_in				<= "00000000";
			via_pb_in(5)			<= '1';
			via_pb_in(6)			<= '1';
		
		else

			
			via_pb_in	<= via_pb_out;
			
			--if via_cb1_oe = '1' then
				via_cb1_in		<= via_cb1_out;
				via_cb1_in		<= '0';
			--end if;
			
						
			if via_pb_oe(3) = '1' then
				ay_bc1			<= via_pb_out(3);
			end if;

			
			if via_pb_oe(4) = '1' then
				ay_bdir        	<= via_pb_out(4);
			end if;

			
			--if via_ca2_oe = '1' then						--nZero , Orange , invertiert
				gpio_0(2)		<= not via_ca2_out;
				via_ca2_in		<= via_ca2_out;
			--else
			--	gpio_0(2)		<= '0';
			--	via_ca2_in		<= '0';
			--end if;
			
			
			--if via_pb_oe(7) = '1' then						--nRamp , Gelb , invertiert
				gpio_0(3)		<= not via_pb_out(7);
				cs_x 			<= not via_pb_out(7);
				cs_y 			<= not via_pb_out(7);
			--end if;
						
			
			--if via_cb2_oe = '1' then						--nBlank, Gruen
				gpio_0(4)		<= via_cb2_out;
				via_cb2_in		<= via_cb2_out;
			--end if;
						
			--via_pb_in(5)	<= gpio_0(5);					--compare , Blau
			
			--if via_pb_oe(0) = '1' then						--sh , Lila
				gpio_0(6)		<= via_pb_out(0);
			--end if;
			
			
			--if via_pb_oe(2) = '1' then						--sel1 , Grau
				gpio_0(7)		<= via_pb_out(2);			
			--end if;
			
			
			--if via_pb_oe(1) = '1' then						--sel0 , Weis
				gpio_0(8)		<= via_pb_out(1);				
			--end if;
			
			

			--demux x channel
			if (via_pb_out(0) = '1') and (via_pb_out(7) = '0') then
				analog_x 	<= analog_vec;
			else
				analog_x 	<= "10000000";
			end if;

			
			--demux y channel			
			if 	(via_pb_out(1) = '0') and (via_pb_out(2) = '0') and (via_pb_out(0) = '0') then
				 analog_y_temp 	<= analog_vec;
			end if;
			
			if (via_pb_out(7) = '0') then
				analog_y 	<= analog_y_temp;
			else
				analog_y 	<= "10000000";
			end if;
			
			--demux z channel
			if 	(via_pb_out(1) = '0') and (via_pb_out(2) = '1') and (via_pb_out(0) = '0') then
				cs_z <= '1';
				analog_z 	<= analog_vec;
			else
				cs_z <= '0';
			end if;
			
			if via_cb2_out = '0' then
				analog_z <= "00000000";
			end if;
			
			--demux offset channel
			if 	(via_pb_out(1) = '1') and (via_pb_out(2) = '0') and (via_pb_out(0) = '0') then
				analog_offs <= analog_vec;
			end if;
			
			
			if via_pa_oe = "00000000" then					--Lese/Schreibe PortA	
				
				   if via_pb_out(3) = '1' then  
				   
				   if ay_data_oe = '0' then
					  via_pa_in		<= ay_data_out;
						   gpio_0(17) 		<= not ay_data_out(7);
						   gpio_0(16) 		<= ay_data_out(6);
						   gpio_0(15) 		<= ay_data_out(5);
						   gpio_0(14) 		<= ay_data_out(4);
						   gpio_0(13) 		<= ay_data_out(3);
						   gpio_0(12) 		<= ay_data_out(2);
						   gpio_0(11) 		<= ay_data_out(1);
						   gpio_0(10) 		<= ay_data_out(0);
						   
						   analog_vec(7) 		<= ay_data_out(7);
						   analog_vec(6) 		<= ay_data_out(6);
						   analog_vec(5) 		<= ay_data_out(5);
						   analog_vec(4) 		<= ay_data_out(4);
						   analog_vec(3) 		<= ay_data_out(3);
						   analog_vec(2) 		<= ay_data_out(2);
						   analog_vec(1) 		<= ay_data_out(1);
						   analog_vec(0) 		<= ay_data_out(0);			--read AY
				   else
				      via_pa_in	    <= via_pa_out;
				      				gpio_0(17) 		<= not via_pa_out(7);
									gpio_0(16) 		<= via_pa_out(6);
									gpio_0(15) 		<= via_pa_out(5);
									gpio_0(14) 		<= via_pa_out(4);
									gpio_0(13) 		<= via_pa_out(3);
									gpio_0(12) 		<= via_pa_out(2);
									gpio_0(11) 		<= via_pa_out(1);
									gpio_0(10) 		<= via_pa_out(0);
									
									analog_vec(7) 		<= via_pa_out(7);
									analog_vec(6) 		<= via_pa_out(6);
									analog_vec(5) 		<= via_pa_out(5);
									analog_vec(4) 		<= via_pa_out(4);
									analog_vec(3) 		<= via_pa_out(3);
									analog_vec(2) 		<= via_pa_out(2);
									analog_vec(1) 		<= via_pa_out(1);
									analog_vec(0) 		<= via_pa_out(0);
				   end if;
				   
--				   gpio_0(17) 		<= not ay_data_out(7);
--				   gpio_0(16) 		<= ay_data_out(6);
--				   gpio_0(15) 		<= ay_data_out(5);
--				   gpio_0(14) 		<= ay_data_out(4);
--				   gpio_0(13) 		<= ay_data_out(3);
--				   gpio_0(12) 		<= ay_data_out(2);
--				   gpio_0(11) 		<= ay_data_out(1);
--				   gpio_0(10) 		<= ay_data_out(0);
--				   
--				   analog_vec(7) 		<= ay_data_out(7);
--				   analog_vec(6) 		<= ay_data_out(6);
--				   analog_vec(5) 		<= ay_data_out(5);
--				   analog_vec(4) 		<= ay_data_out(4);
--				   analog_vec(3) 		<= ay_data_out(3);
--				   analog_vec(2) 		<= ay_data_out(2);
--				   analog_vec(1) 		<= ay_data_out(1);
--				   analog_vec(0) 		<= ay_data_out(0);
				   
				   else
				   via_pa_in		<= analog_x;--via_pa_out;		--read PortA , DA hat kein Latch daher 1 (NMOS)
				   
				   gpio_0(17) 		<= '0';--not via_pa_out(7);
				   gpio_0(16) 		<= '1';--via_pa_out(6);
				   gpio_0(15) 		<= '1';--via_pa_out(5);
				   gpio_0(14) 		<= '1';--via_pa_out(4);
				   gpio_0(13) 		<= '1';--via_pa_out(3);
				   gpio_0(12) 		<= '1';--via_pa_out(2);
				   gpio_0(11) 		<= '1';--via_pa_out(1);
				   gpio_0(10) 		<= '1';--via_pa_out(0);
				   
				   analog_vec(7) 		<= '1';--not via_pa_out(7);
				   analog_vec(6) 		<= '1';--via_pa_out(6);
				   analog_vec(5) 		<= '1';--via_pa_out(5);
				   analog_vec(4) 		<= '1';--via_pa_out(4);
				   analog_vec(3) 		<= '1';--via_pa_out(3);
				   analog_vec(2) 		<= '1';--via_pa_out(2);
				   analog_vec(1) 		<= '1';--via_pa_out(1);
				   analog_vec(0) 		<= '1';--via_pa_out(0);
				   
				end if;
			else					
				via_pa_in		<= via_pa_out;				--write PortA
				ay_data_in		<= via_pa_out;				
				
				gpio_0(17) 		<= not via_pa_out(7);
				gpio_0(16) 		<= via_pa_out(6);
				gpio_0(15) 		<= via_pa_out(5);
				gpio_0(14) 		<= via_pa_out(4);
				gpio_0(13) 		<= via_pa_out(3);
				gpio_0(12) 		<= via_pa_out(2);
				gpio_0(11) 		<= via_pa_out(1);
				gpio_0(10) 		<= via_pa_out(0);
				
				analog_vec(7) 		<= via_pa_out(7);
				analog_vec(6) 		<= via_pa_out(6);
				analog_vec(5) 		<= via_pa_out(5);
				analog_vec(4) 		<= via_pa_out(4);
				analog_vec(3) 		<= via_pa_out(3);
				analog_vec(2) 		<= via_pa_out(2);
				analog_vec(1) 		<= via_pa_out(1);
				analog_vec(0) 		<= via_pa_out(0);
			end if;		
			
--	end if;
end if;
	
end process;

reset : process begin 
   wait until rising_edge(clk1m5) and pll_locked = '1';
   resetsr <= resetsr(0) & ares;
end process;

vex_clk : process (cpu_clk,clk6m,clk1m5)
begin
	if rising_edge(cpu_clk) then
		if cnt6m = 2-1 then
		   clk6m  <= '1';
		   cnt6m  <= 0;
		else
		   clk6m  <= '0';
		   cnt6m  <= cnt6m +1;
		end if;
	 end if; 
   
	if rising_edge(clk6m) then       
		if cnt1m5  = 4-1 then
		   clk1m5  <= '1';
		   cnt1m5  <= 0;
		else
		   clk1m5  <= '0';
		   cnt1m5  <= cnt1m5 +1;
		end if;
	 end if; 
	 
  if rising_edge(clk1m5) then       
		if cnt750k  = 2-1 then
		   clk750k  <= '1';
		   cnt750k  <= 0;
		else
		   clk750k  <= '0';
		   cnt750k  <= cnt750k +1;
		end if;
  end if;  	  
end process;


end my_computer; --===================== End of architecture =======================--

