library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

use work.pkg_vectrex.all;
use work.pkg_d2a.all;

entity vectrex_xlink is
    Port ( O_PWM_XH : out  STD_LOGIC;
           O_PWM_XL : out  STD_LOGIC;
           O_PWM_YH : out  STD_LOGIC;
           O_PWM_YL : out  STD_LOGIC;
           O_VIDEO_Z : out  STD_LOGIC_VECTOR(3 downto 0);
			  I_PSXDATA				  : in    std_logic;
			  O_PSXCMD				  : out   std_logic;
			  O_PSXATT				  : out   std_logic;
           O_PSXCLK				  : out   std_logic;
	        I_PSXACK				  : in    std_logic;
           I_CLK : in  STD_LOGIC;
           I_RESET : in  STD_LOGIC;
           O_AUDIO_L : out  STD_LOGIC;
           O_AUDIO_R : out  STD_LOGIC);
end vectrex_xlink;


architecture Behavioral of vectrex_xlink is
signal buttons:		STD_LOGIC_VECTOR(7 downto 0);
signal p1x:				STD_LOGIC_VECTOR(7 downto 0);
signal p1y:				STD_LOGIC_VECTOR(7 downto 0);
signal p2x:				STD_LOGIC_VECTOR(7 downto 0);
signal p2y:				STD_LOGIC_VECTOR(7 downto 0);
signal psxbuttons:	STD_LOGIC_VECTOR(15 downto 0);
signal clk_ibuf:		STD_LOGIC;
signal clk_dlla_op0:	STD_LOGIC;
signal clk_dlla_div:	STD_LOGIC;
signal clk_dlla_2x:	STD_LOGIC;
signal reset_h:		STD_LOGIC;
signal reset_l:		STD_LOGIC;
signal clk_104:		STD_LOGIC;
signal clk_12:			STD_LOGIC;
signal clke_psx:		STD_LOGIC;
signal clkc_psx:		STD_LOGIC_VECTOR(3 downto 0);
signal clk_pwm:		STD_LOGIC;
signal xval:			STD_LOGIC_VECTOR(9 downto 0);
signal yval:			STD_LOGIC_VECTOR(9 downto 0);
signal xval_filt:			STD_LOGIC_VECTOR(9 downto 0);
signal yval_filt:			STD_LOGIC_VECTOR(9 downto 0);
signal xval_pwm:			STD_LOGIC_VECTOR(9 downto 0);
signal yval_pwm:			STD_LOGIC_VECTOR(9 downto 0);
signal xval_inv:			STD_LOGIC_VECTOR(9 downto 0);
signal yval_inv:			STD_LOGIC_VECTOR(9 downto 0);
signal zval:			STD_LOGIC_VECTOR(7 downto 0);
signal soundval:			STD_LOGIC_VECTOR(7 downto 0);
signal soundout:			STD_LOGIC;
signal cpu_reset:		STD_LOGIC;

attribute CLKDV_DIVIDE : string;
attribute CLKDV_DIVIDE of CLKDLLA : label is "4.5";
begin
	IBUFG0 : IBUFG port map (I=>I_CLK,  O=>clk_ibuf);
	CLKDLLA : CLKDLL port map (
		CLKIN  => clk_ibuf,
		RST    => reset_h,
		CLKFB  => clk_dlla_op0,
		CLK0   => clk_dlla_op0,
		CLKDV  => clk_dlla_div,
		CLK2X  => clk_dlla_2x
	);
	BUFG0 : BUFG port map (I=> clk_dlla_op0,O => clk_104);
	BUFG1 : BUFG port map (I=> clk_dlla_div,O => clk_12);
	BUFG2 : BUFG port map (I=> clk_dlla_2x,O => clk_pwm);

	pwmxh: pwmgen port map (
		inval => xval_pwm(9 downto 5),
		output => O_PWM_XH,
		clk => clk_pwm,
		reset => reset_h
	);
	pwmxl: pwmgen port map (
		inval => xval_pwm(4 downto 0),
		output => O_PWM_XL,
		clk => clk_pwm,
		reset => reset_h
	);
	pwmyh: pwmgen port map (
		inval => yval_pwm(9 downto 5),
		output => O_PWM_YH,
		clk => clk_pwm,
		reset => reset_h
	);
	pwmyl: pwmgen port map (
		inval => yval_pwm(4 downto 0),
		output => O_PWM_YL,
		clk => clk_pwm,
		reset => reset_h
	);
	
--	noisex: noisegen port map (
--		noise => xval,
--		clock => clk_12,
--		reset => reset_h
--	);
--	O_VIDEO_Z<="0010";

	filterx: filter generic map (
			mul => 1
		) port map (
			input => xval_inv,
			output => xval_filt,
			clk => clk_12,
			reset => reset_h
	);
	filtery: filter generic map (
			mul => 6
		) port map (
			input => yval_inv,
			output => yval_filt,
			clk => clk_12,
			reset => reset_h
	);
	audioda: deltasigma generic map (
			width => 8
		) port map (
			inval => soundval,
			output => soundout,
			clk => clk_pwm,
			reset => reset_h
	);



	myvectrex: vectrex port map (
		cpu_areset => cpu_reset,
		cpu_ahold => '0',
		cpu_anmi => '0',
		cpu_clk => clk_12,
		analog_sound_out => soundval,
		analog_x_out => yval,
		analog_y_out => xval,
		analog_z_out => zval,
		p1_x => p1x,
		p1_y => p1y,
		p2_x => p2x,
		p2_y => p2y,
		p1_buttons => buttons(3 downto 0),
		p2_buttons => buttons(7 downto 4)
	);

	psxclk: process(clk_12, reset_h)
	begin
		if (reset_h='1') then
			clkc_psx<="0000";
		elsif rising_edge(clk_12) then
			clkc_psx<=clkc_psx+"0001";
			if (clkc_psx="0000") then
				clke_psx<='1';
			else
				clke_psx<='0';
			end if;
		end if;
	end process;

	psxcontroller : entity work.psxcontroller
		port map (
			psxdata => I_PSXDATA,
			psxcmd => O_PSXCMD,
			psxatt => O_PSXATT,
			psxclk => O_PSXCLK,
			psxack => I_PSXACK,
			buttons => psxbuttons,
			clk => clk_12,
			clken => clke_psx,
			reset_l => reset_l
		);

	--psxbuttons(15 downto 0): SQUARE CROSS CIRCLE TRIANGLE R1 L1 R2 L2 LEFT DOWN RIGHT UP START X X SELECT
	buttons(7 downto 4)<="1111";
	buttons(3)<=not psxbuttons(12);
	buttons(2)<=not psxbuttons(13);
	buttons(1)<=not psxbuttons(14);
	buttons(0)<=not psxbuttons(15);
	
	p2x<="01111111";
	p2y<="01111111";
	
	p1x<="01111111" when psxbuttons(7)='1' else
		  "10000000" when psxbuttons(5)='1' else
		  "00000000";
	p1y<="01111111" when psxbuttons(6)='1' else
		  "10000000" when psxbuttons(4)='1' else
		  "00000000";
	
	cpu_reset<=reset_h or psxbuttons(3);
	reset_h<=I_RESET;
	reset_l<=not I_RESET;
--	xval_inv<="1111111111"-xval;
	xval_inv<=("1111111111"-xval) xor "1000000000";
	yval_inv<=yval xor "1000000000";
	
	yval_pwm<=yval_inv when psxbuttons(8)='1' else yval_filt;
	xval_pwm<=xval_inv when psxbuttons(8)='1' else xval_filt;

	O_VIDEO_Z<=zval(6 downto 3) when zval(7)='0' else "0000";

	O_AUDIO_L<=soundout;
	O_AUDIO_R<=soundout;

end Behavioral;
